@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 from plone.app.caching.interfaces import _
 from plone.app.caching.operations.utils import cachedResponse
 from plone.app.caching.operations.utils import cacheInRAM
@@ -31,7 +30,7 @@ import time
 @implementer(ICachingOperation)
 @provider(ICachingOperationType)
 @adapter(Interface, Interface)
-class BaseCaching(object):
+class BaseCaching:
     """A generic caching operation class that can do pretty much all the usual
     caching operations based on options settings. For UI simplicity, it might
     be easier to subclass this in your custom operations to set a few default
@@ -61,17 +60,24 @@ class BaseCaching(object):
         is a string to add as a Vary header value in the response.
     """
 
-    title = _(u'Generic caching')
+    title = _("Generic caching")
     description = _(
-        u'Through this operation, all standard caching functions '
-        u'can be performed via various combinations of the optional '
-        u'parameter settings. For most cases, it\'s probably easier '
-        u'to use one of the other simpler operations (Strong caching, '
-        u'Moderate caching, Weak caching, or No caching).'
+        "Through this operation, all standard caching functions "
+        "can be performed via various combinations of the optional "
+        "parameter settings. For most cases, it's probably easier "
+        "to use one of the other simpler operations (Strong caching, "
+        "Moderate caching, Weak caching, or No caching)."
+    )
+    prefix = "plone.app.caching.baseCaching"
+    options = (
+        "maxage",
+        "smaxage",
+        "etags",
+        "lastModified",
+        "ramCache",
+        "vary",
+        "anonOnly",
     )
-    prefix = 'plone.app.caching.baseCaching'
-    options = ('maxage', 'smaxage', 'etags', 'lastModified',
-               'ramCache', 'vary', 'anonOnly')
 
     # Default option values
     maxage = smaxage = etags = vary = None
@@ -84,26 +90,27 @@ class BaseCaching(object):
     def interceptResponse(self, rulename, response, class_=None):
         options = lookupOptions(class_ or self.__class__, rulename)
 
-        etags = options.get('etags') or self.etags
-        anonOnly = options.get('anonOnly', self.anonOnly)
-        ramCache = options.get('ramCache', self.ramCache)
-        lastModified = options.get('lastModified', self.lastModified)
+        etags = options.get("etags") or self.etags
+        anonOnly = options.get("anonOnly", self.anonOnly)
+        ramCache = options.get("ramCache", self.ramCache)
+        lastModified = options.get("lastModified", self.lastModified)
 
         # Add the ``anonymousOrRandom`` etag if we are anonymous only
         if anonOnly:
             if etags is None:
-                etags = ['anonymousOrRandom']
-            elif 'anonymousOrRandom' not in etags:
-                etags = tuple(etags) + ('anonymousOrRandom',)
+                etags = ["anonymousOrRandom"]
+            elif "anonymousOrRandom" not in etags:
+                etags = tuple(etags) + ("anonymousOrRandom",)
 
         etag = getETagAnnotation(self.published, self.request, keys=etags)
         lastModified = getLastModifiedAnnotation(
-            self.published, self.request, lastModified=lastModified)
+            self.published, self.request, lastModified=lastModified
+        )
 
         # Remove Range from request if the If-Range condition is not fulfilled
-        if_range = self.request.environ.get('HTTP_IF_RANGE', '').strip('"')
+        if_range = self.request.environ.get("HTTP_IF_RANGE", "").strip('"')
         if if_range:
-            if 'HTTP_RANGE' in self.request.environ:
+            if "HTTP_RANGE" in self.request.environ:
                 if_range_dt = parseDateTime(if_range)
                 delta_sec = datetime.timedelta(seconds=1)
                 if if_range_dt and (lastModified - if_range_dt) < delta_sec:
@@ -111,13 +118,13 @@ class BaseCaching(object):
                 elif if_range == etag:
                     pass
                 else:
-                    del self.request.environ['HTTP_RANGE']
+                    del self.request.environ["HTTP_RANGE"]
             # If-Range check is done here so we could remove it from the request
-            del self.request.environ['HTTP_IF_RANGE']
+            del self.request.environ["HTTP_IF_RANGE"]
 
         # Check for cache stop request variables
         if cacheStop(self.request, rulename):
-            return None
+            return
 
         # Check if this should be a 304 response
         if not isModified(self.request, etag=etag, lastModified=lastModified):
@@ -133,48 +140,46 @@ class BaseCaching(object):
         if ramCache:
             context = getContext(self.published)
             portal_state = getMultiAdapter(
-                (context, self.request), name=u'plone_portal_state')
+                (context, self.request), name="plone_portal_state"
+            )
 
             if portal_state.anonymous():
                 cached = fetchFromRAMCache(
-                    self.request, etag=etag, lastModified=lastModified)
+                    self.request, etag=etag, lastModified=lastModified
+                )
                 if cached is not None:
                     return cachedResponse(
-                        self.published,
-                        self.request,
-                        response,
-                        *cached
+                        self.published, self.request, response, *cached
                     )
 
-        return None
-
     def modifyResponse(self, rulename, response, class_=None):
         options = lookupOptions(class_ or self.__class__, rulename)
 
-        maxage = options.get('maxage', self.maxage)
-        smaxage = options.get('smaxage', self.smaxage)
-        etags = options.get('etags') or self.etags
+        maxage = options.get("maxage", self.maxage)
+        smaxage = options.get("smaxage", self.smaxage)
+        etags = options.get("etags") or self.etags
 
-        anonOnly = options.get('anonOnly', self.anonOnly)
-        ramCache = options.get('ramCache', self.ramCache)
-        vary = options.get('vary', self.vary)
+        anonOnly = options.get("anonOnly", self.anonOnly)
+        ramCache = options.get("ramCache", self.ramCache)
+        vary = options.get("vary", self.vary)
 
         # Add the ``anonymousOrRandom`` etag if we are anonymous only
         if anonOnly:
             if etags is None:
-                etags = ['anonymousOrRandom']
-            elif 'anonymousOrRandom' not in etags:
-                etags = tuple(etags) + ('anonymousOrRandom',)
+                etags = ["anonymousOrRandom"]
+            elif "anonymousOrRandom" not in etags:
+                etags = tuple(etags) + ("anonymousOrRandom",)
 
         etag = getETagAnnotation(self.published, self.request, etags)
         lastModified = getLastModifiedAnnotation(
-            self.published, self.request, options['lastModified'])
+            self.published, self.request, options["lastModified"]
+        )
 
         # Check for cache stop request variables
         if cacheStop(self.request, rulename):
             # only stop with etags if configured
             if etags:
-                etag = '{0}{1}'.format(time.time(), random.randint(0, 1000))
+                etag = f"{time.time()}{random.randint(0, 1000)}"
                 return setCacheHeaders(
                     self.published,
                     self.request,
@@ -193,18 +198,17 @@ class BaseCaching(object):
         if ramCache or proxyCache:
             if etags is not None:
                 if (
-                    'userid' in etags or
-                    'anonymousOrRandom' in etags or
-                    'roles' in etags
-
+                    "userid" in etags
+                    or "anonymousOrRandom" in etags
+                    or "roles" in etags
                 ):
                     context = getContext(self.published)
                     portal_state = getMultiAdapter(
                         (context, self.request),
-                        name=u'plone_portal_state',
+                        name="plone_portal_state",
                     )
                     public = portal_state.anonymous()
-            public = public and visibleToRole(self.published, role='Anonymous')
+            public = public and visibleToRole(self.published, role="Anonymous")
 
         if proxyCache and not public:
             # This is private so keep it out of both shared and browser caches
@@ -222,8 +226,13 @@ class BaseCaching(object):
         )
 
         if ramCache and public:
-            cacheInRAM(self.published, self.request, response,
-                       etag=etag, lastModified=lastModified)
+            cacheInRAM(
+                self.published,
+                self.request,
+                response,
+                etag=etag,
+                lastModified=lastModified,
+            )
 
 
 @provider(ICachingOperationType)
@@ -232,21 +241,21 @@ class WeakCaching(BaseCaching):
     operation to help make the UI approachable by mortals
     """
 
-    title = _(u'Weak caching')
+    title = _("Weak caching")
     description = _(
-        u'Cache in browser but expire immediately and enable 304 '
-        u'responses on subsequent requests. 304\'s require configuration '
-        u'of the \'Last-modified\' and/or \'ETags\' settings. If '
-        u'Last-Modified  header is insufficient to ensure freshness, turn on '
-        u'ETag checking by listing each ETag components that should be used '
-        u'to construct the ETag header. To also cache public responses in '
-        u'Zope memory, set \'RAM cache\' to True.'
+        "Cache in browser but expire immediately and enable 304 "
+        "responses on subsequent requests. 304's require configuration "
+        "of the 'Last-modified' and/or 'ETags' settings. If "
+        "Last-Modified  header is insufficient to ensure freshness, turn on "
+        "ETag checking by listing each ETag components that should be used "
+        "to construct the ETag header. To also cache public responses in "
+        "Zope memory, set 'RAM cache' to True."
     )
-    prefix = 'plone.app.caching.weakCaching'
+    prefix = "plone.app.caching.weakCaching"
     sort = 3
 
     # Configurable options
-    options = ('etags', 'lastModified', 'ramCache', 'vary', 'anonOnly')
+    options = ("etags", "lastModified", "ramCache", "vary", "anonOnly")
 
     # Default option values
     maxage = 0
@@ -260,20 +269,20 @@ class ModerateCaching(BaseCaching):
     operation to help make the UI approachable by mortals
     """
 
-    title = _(u'Moderate caching')
+    title = _("Moderate caching")
     description = _(
-        u'Cache in browser but expire immediately (same as \'weak caching\'), '
-        u'and cache in proxy (default: 24 hrs). '
-        u'Use a purgable caching reverse proxy for best results. '
-        u'Caution: If proxy cannot be purged, or cannot be configured '
-        u'to remove the \'s-maxage\' token from the response, then stale '
-        u'responses might be seen until the cached entry expires.')
-    prefix = 'plone.app.caching.moderateCaching'
+        "Cache in browser but expire immediately (same as 'weak caching'), "
+        "and cache in proxy (default: 24 hrs). "
+        "Use a purgable caching reverse proxy for best results. "
+        "Caution: If proxy cannot be purged, or cannot be configured "
+        "to remove the 's-maxage' token from the response, then stale "
+        "responses might be seen until the cached entry expires."
+    )
+    prefix = "plone.app.caching.moderateCaching"
     sort = 2
 
     # Configurable options
-    options = ('smaxage', 'etags', 'lastModified',
-               'ramCache', 'vary', 'anonOnly')
+    options = ("smaxage", "etags", "lastModified", "ramCache", "vary", "anonOnly")
 
     # Default option values
     maxage = 0
@@ -288,19 +297,26 @@ class StrongCaching(BaseCaching):
     operation to help make the UI approachable by mortals
     """
 
-    title = _(u'Strong caching')
+    title = _("Strong caching")
     description = _(
-        u'Cache in browser and proxy (default: 24 hrs). '
-        u'Caution: Only use for stable resources '
-        u'that never change without changing their URL, or resources '
-        u'for which temporary staleness is not critical.'
+        "Cache in browser and proxy (default: 24 hrs). "
+        "Caution: Only use for stable resources "
+        "that never change without changing their URL, or resources "
+        "for which temporary staleness is not critical."
     )
-    prefix = 'plone.app.caching.strongCaching'
+    prefix = "plone.app.caching.strongCaching"
     sort = 1
 
     # Configurable options
-    options = ('maxage', 'smaxage', 'etags', 'lastModified',
-               'ramCache', 'vary', 'anonOnly')
+    options = (
+        "maxage",
+        "smaxage",
+        "etags",
+        "lastModified",
+        "ramCache",
+        "vary",
+        "anonOnly",
+    )
 
     # Default option values
     maxage = 86400
@@ -308,19 +324,54 @@ class StrongCaching(BaseCaching):
     lastModified = ramCache = anonOnly = False
 
 
+@provider(ICachingOperationType)
+class TerseCaching(BaseCaching):
+    """Terse caching operation."""
+
+    title = _("Terse caching")
+    description = _(
+        "Cache in browser (default: 10s) and proxy (default: 1 min). "
+        "Keep very short in cache to buffer hi-load scenarios ",
+        "Use for very dynamic content, like collecting data from "
+        "the whole portal. "
+        "Temporary outdated is less important than performance."
+        "Reduce in-cache time to just 3-10 seconds to only buffer peak load "
+        "if behind a proxy",
+    )
+    prefix = "plone.app.caching.terseCaching"
+    sort = 4
+
+    # Configurable options
+    options = (
+        "maxage",
+        "smaxage",
+        "etags",
+        "lastModified",
+        "ramCache",
+        "vary",
+        "anonOnly",
+    )
+
+    # Default option values
+    maxage = 10
+    smaxage = 60
+    vary = "Accept"
+    etags = None
+    lastModified = ramCache = anonOnly = False
+
+
 @implementer(ICachingOperation)
 @provider(ICachingOperationType)
 @adapter(Interface, IHTTPRequest)
-class NoCaching(object):
+class NoCaching:
     """A caching operation that tries to keep the response
     out of all caches.
     """
 
-    title = _(u'No caching')
-    description = _(u'Use this operation to keep the response '
-                    u'out of all caches.')
-    prefix = 'plone.app.caching.noCaching'
-    sort = 4
+    title = _("No caching")
+    description = _("Use this operation to keep the response " "out of all caches.")
+    prefix = "plone.app.caching.noCaching"
+    sort = 5
     options = ()
 
     def __init__(self, published, request):
@@ -328,7 +379,7 @@ class NoCaching(object):
         self.request = request
 
     def interceptResponse(self, rulename, response):
-        return None
+        return
 
     def modifyResponse(self, rulename, response):
         doNotCache(self.published, self.request, response)
