@@ -1,14 +1,15 @@
-# -*- coding: utf-8 -*-
 from Acquisition import Explicit
 from datetime import datetime
 from dateutil.tz import tzlocal
+from io import StringIO
 from plone.registry.fieldfactory import persistentFieldAdapter
 from plone.testing.zca import UNIT_TESTING
 from Products.CMFCore.interfaces import IContentish
-from six import StringIO
+from Products.CMFCore.interfaces import IMembershipTool
 from z3c.caching.interfaces import ILastModified
 from zope.component import adapter
 from zope.component import provideAdapter
+from zope.component import provideUtility
 from zope.interface import implementer
 from zope.interface import Interface
 from ZPublisher.HTTPRequest import HTTPRequest
@@ -20,11 +21,11 @@ import unittest
 
 @implementer(IContentish)
 class DummyContext(Explicit):
-    pass
+    def getLayout(self):
+        return getattr(self, "_layout", None)
 
 
-class DummyPublished(object):
-
+class DummyPublished:
     def __init__(self, parent=None):
         self.__parent__ = parent
 
@@ -41,19 +42,18 @@ class TestETags(unittest.TestCase):
     def test_UserID_anonymous(self):
         from plone.app.caching.operations.etags import UserID
 
-        @implementer(Interface)
-        @adapter(DummyContext, Interface)
-        class DummyPortalState(object):
-
-            def __init__(self, context, request):
+        @adapter(Interface, Interface)
+        @implementer(IMembershipTool)
+        class DummyMembershipTool:
+            def __init__(self):
                 pass
 
-            def member(self):
-                return None
+            def getAuthenticatedMember(self):
+                return
 
-        provideAdapter(DummyPortalState, name=u'plone_portal_state')
+        provideUtility(DummyMembershipTool())
 
-        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
+        environ = {"SERVER_NAME": "example.com", "SERVER_PORT": "80"}
         response = HTTPResponse()
         request = HTTPRequest(StringIO(), environ, response)
         published = DummyPublished(DummyContext())
@@ -65,153 +65,138 @@ class TestETags(unittest.TestCase):
     def test_UserID_member(self):
         from plone.app.caching.operations.etags import UserID
 
-        class DummyMember(object):
-
+        class DummyMember:
             def getId(self):
-                return 'bob'
+                return "bob"
 
-        @implementer(Interface)
-        @adapter(DummyContext, Interface)
-        class DummyPortalState(object):
-
-            def __init__(self, context, request):
+        @adapter(Interface, Interface)
+        @implementer(IMembershipTool)
+        class DummyMembershipTool:
+            def __init__(self):
                 pass
 
-            def member(self):
+            def getAuthenticatedMember(self):
                 return DummyMember()
 
-        provideAdapter(DummyPortalState, name=u'plone_portal_state')
+        provideUtility(DummyMembershipTool())
 
-        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
+        environ = {"SERVER_NAME": "example.com", "SERVER_PORT": "80"}
         response = HTTPResponse()
         request = HTTPRequest(StringIO(), environ, response)
         published = DummyPublished(DummyContext())
 
         etag = UserID(published, request)
 
-        self.assertEqual('bob', etag())
+        self.assertEqual("bob", etag())
 
     # Roles
 
     def test_Roles_anonymous(self):
         from plone.app.caching.operations.etags import Roles
 
-        @implementer(Interface)
-        @adapter(DummyContext, Interface)
-        class DummyPortalState(object):
-
-            def __init__(self, context, request):
+        @adapter(Interface, Interface)
+        @implementer(IMembershipTool)
+        class DummyMembershipTool:
+            def __init__(self):
                 pass
 
-            def anonymous(self):
+            def getAuthenticatedMember(self):
+                return
+
+            def isAnonymousUser(self):
                 return True
 
-            def member(self):
-                return None
+        provideUtility(DummyMembershipTool())
 
-        provideAdapter(DummyPortalState, name=u'plone_portal_state')
-
-        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
+        environ = {"SERVER_NAME": "example.com", "SERVER_PORT": "80"}
         response = HTTPResponse()
         request = HTTPRequest(StringIO(), environ, response)
         published = DummyPublished(DummyContext())
 
         etag = Roles(published, request)
 
-        self.assertEqual('Anonymous', etag())
+        self.assertEqual("Anonymous", etag())
 
     def test_Roles_member(self):
         from plone.app.caching.operations.etags import Roles
 
-        class DummyMember(object):
-
+        class DummyMember:
             def getRolesInContext(self, context):
-                return ['Member', 'Manager']
+                return ["Member", "Manager"]
 
-        @implementer(Interface)
-        @adapter(DummyContext, Interface)
-        class DummyPortalState(object):
-
-            def __init__(self, context, request):
+        @adapter(Interface, Interface)
+        @implementer(IMembershipTool)
+        class DummyMembershipTool:
+            def __init__(self):
                 pass
 
-            def anonymous(self):
+            def isAnonymousUser(self):
                 return False
 
-            def member(self):
+            def getAuthenticatedMember(self):
                 return DummyMember()
 
-        provideAdapter(DummyPortalState, name=u'plone_portal_state')
+        provideUtility(DummyMembershipTool())
 
-        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
+        environ = {"SERVER_NAME": "example.com", "SERVER_PORT": "80"}
         response = HTTPResponse()
         request = HTTPRequest(StringIO(), environ, response)
         published = DummyPublished(DummyContext())
 
         etag = Roles(published, request)
 
-        self.assertEqual('Manager;Member', etag())
+        self.assertEqual("Manager;Member", etag())
 
     # Language
 
     def test_Language_no_header(self):
         from plone.app.caching.operations.etags import Language
 
-        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
+        environ = {"SERVER_NAME": "example.com", "SERVER_PORT": "80"}
         response = HTTPResponse()
         request = HTTPRequest(StringIO(), environ, response)
         published = DummyPublished(DummyContext())
 
         etag = Language(published, request)
 
-        self.assertEqual('', etag())
+        self.assertEqual("", etag())
 
     def test_Language_with_header(self):
         from plone.app.caching.operations.etags import Language
 
-        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
+        environ = {"SERVER_NAME": "example.com", "SERVER_PORT": "80"}
         response = HTTPResponse()
         request = HTTPRequest(StringIO(), environ, response)
         published = DummyPublished(DummyContext())
 
-        request.environ['HTTP_ACCEPT_LANGUAGE'] = 'en'
+        request.environ["HTTP_ACCEPT_LANGUAGE"] = "en"
 
         etag = Language(published, request)
 
-        self.assertEqual('en', etag())
+        self.assertEqual("en", etag())
 
     # UserLanguage
 
     def test_UserLanguage(self):
         from plone.app.caching.operations.etags import UserLanguage
 
-        @implementer(Interface)
-        @adapter(DummyContext, Interface)
-        class DummyPortalState(object):
-
-            def __init__(self, context, request):
-                pass
-
-            def language(self):
-                return 'en'
-
-        provideAdapter(DummyPortalState, name=u'plone_portal_state')
-
-        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
+        environ = {"SERVER_NAME": "example.com", "SERVER_PORT": "80"}
         response = HTTPResponse()
         request = HTTPRequest(StringIO(), environ, response)
-        published = DummyPublished(DummyContext())
+        context = DummyContext()
+        context.Language = lambda: "en"
+        published = DummyPublished(context)
 
         etag = UserLanguage(published, request)
 
-        self.assertEqual('en', etag())
+        self.assertEqual("en", etag())
 
     # LastModified
 
     def test_LastModified_no_adapter(self):
         from plone.app.caching.operations.etags import LastModified
 
-        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
+        environ = {"SERVER_NAME": "example.com", "SERVER_PORT": "80"}
         response = HTTPResponse()
         request = HTTPRequest(StringIO(), environ, response)
         published = DummyPublished(DummyContext())
@@ -225,17 +210,16 @@ class TestETags(unittest.TestCase):
 
         @implementer(ILastModified)
         @adapter(DummyPublished)
-        class DummyLastModified(object):
-
+        class DummyLastModified:
             def __init__(self, context):
                 self.context = context
 
             def __call__(self):
-                return None
+                return
 
         provideAdapter(DummyLastModified)
 
-        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
+        environ = {"SERVER_NAME": "example.com", "SERVER_PORT": "80"}
         response = HTTPResponse()
         request = HTTPRequest(StringIO(), environ, response)
         published = DummyPublished(DummyContext())
@@ -252,8 +236,7 @@ class TestETags(unittest.TestCase):
 
         @implementer(ILastModified)
         @adapter(DummyPublished)
-        class DummyLastModified(object):
-
+        class DummyLastModified:
             def __init__(self, context):
                 self.context = context
 
@@ -262,7 +245,7 @@ class TestETags(unittest.TestCase):
 
         provideAdapter(DummyLastModified)
 
-        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
+        environ = {"SERVER_NAME": "example.com", "SERVER_PORT": "80"}
         response = HTTPResponse()
         request = HTTPRequest(StringIO(), environ, response)
         published = DummyPublished(DummyContext())
@@ -274,32 +257,23 @@ class TestETags(unittest.TestCase):
 
     def test_CatalogCounter(self):
         from plone.app.caching.operations.etags import CatalogCounter
+        from Products.CMFCore.interfaces import ICatalogTool
 
-        class DummyCatalog(object):
-
+        @implementer(ICatalogTool)
+        class DummyCatalog:
             def getCounter(self):
                 return 10
 
-        @implementer(Interface)
-        @adapter(DummyContext, Interface)
-        class DummyTools(object):
+        provideUtility(DummyCatalog())
 
-            def __init__(self, context, request):
-                pass
-
-            def catalog(self):
-                return DummyCatalog()
-
-        provideAdapter(DummyTools, name=u'plone_tools')
-
-        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
+        environ = {"SERVER_NAME": "example.com", "SERVER_PORT": "80"}
         response = HTTPResponse()
         request = HTTPRequest(StringIO(), environ, response)
         published = DummyPublished(DummyContext())
 
         etag = CatalogCounter(published, request)
 
-        self.assertEqual('10', etag())
+        self.assertEqual("10", etag())
 
     # ObjectLocked
 
@@ -308,92 +282,116 @@ class TestETags(unittest.TestCase):
 
         @implementer(Interface)
         @adapter(DummyContext, Interface)
-        class DummyContextState(object):
-
+        class DummyContextState:
             def __init__(self, context, request):
                 pass
 
             def is_locked(self):
                 return True
 
-        provideAdapter(DummyContextState, name=u'plone_context_state')
+        provideAdapter(DummyContextState, name="plone_context_state")
 
-        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
+        environ = {"SERVER_NAME": "example.com", "SERVER_PORT": "80"}
         response = HTTPResponse()
         request = HTTPRequest(StringIO(), environ, response)
         published = DummyPublished(DummyContext())
 
         etag = ObjectLocked(published, request)
 
-        self.assertEqual('1', etag())
+        self.assertEqual("1", etag())
 
     def test_ObjectLocked_false(self):
         from plone.app.caching.operations.etags import ObjectLocked
 
         @implementer(Interface)
         @adapter(DummyContext, Interface)
-        class DummyContextState(object):
-
+        class DummyContextState:
             def __init__(self, context, request):
                 pass
 
             def is_locked(self):
                 return False
 
-        provideAdapter(DummyContextState, name=u'plone_context_state')
+        provideAdapter(DummyContextState, name="plone_context_state")
 
-        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
+        environ = {"SERVER_NAME": "example.com", "SERVER_PORT": "80"}
         response = HTTPResponse()
         request = HTTPRequest(StringIO(), environ, response)
         published = DummyPublished(DummyContext())
 
         etag = ObjectLocked(published, request)
 
-        self.assertEqual('0', etag())
+        self.assertEqual("0", etag())
 
     # Skin
 
     def test_Skin_request_variable(self):
         from plone.app.caching.operations.etags import Skin
 
-        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
+        environ = {"SERVER_NAME": "example.com", "SERVER_PORT": "80"}
         response = HTTPResponse()
         request = HTTPRequest(StringIO(), environ, response)
         published = DummyPublished(DummyContext())
 
-        class DummyPortalSkins(object):
-
+        class DummyPortalSkins:
             def getRequestVarname(self):
-                return 'skin_name'
+                return "skin_name"
 
             def getDefaultSkin(self):
-                return 'defaultskin'
+                return "defaultskin"
 
         published.__parent__.portal_skins = DummyPortalSkins()
-        request['skin_name'] = 'otherskin'
+        request["skin_name"] = "otherskin"
 
         etag = Skin(published, request)
 
-        self.assertEqual('otherskin', etag())
+        self.assertEqual("otherskin", etag())
 
     def test_Skin_default(self):
         from plone.app.caching.operations.etags import Skin
 
-        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
+        environ = {"SERVER_NAME": "example.com", "SERVER_PORT": "80"}
         response = HTTPResponse()
         request = HTTPRequest(StringIO(), environ, response)
         published = DummyPublished(DummyContext())
 
-        class DummyPortalSkins(object):
-
+        class DummyPortalSkins:
             def getRequestVarname(self):
-                return 'skin_name'
+                return "skin_name"
 
             def getDefaultSkin(self):
-                return 'defaultskin'
+                return "defaultskin"
 
         published.__parent__.portal_skins = DummyPortalSkins()
 
         etag = Skin(published, request)
 
-        self.assertEqual('defaultskin', etag())
+        self.assertEqual("defaultskin", etag())
+
+    # layout
+
+    def test_layout_none(self):
+        from plone.app.caching.operations.etags import Layout
+
+        environ = {"SERVER_NAME": "example.com", "SERVER_PORT": "80"}
+        response = HTTPResponse()
+        request = HTTPRequest(StringIO(), environ, response)
+        published = DummyPublished(DummyContext())
+
+        etag = Layout(published, request)
+
+        self.assertIsNone(etag())
+
+    def test_layout_set(self):
+        from plone.app.caching.operations.etags import Layout
+
+        environ = {"SERVER_NAME": "example.com", "SERVER_PORT": "80"}
+        response = HTTPResponse()
+        request = HTTPRequest(StringIO(), environ, response)
+        context = DummyContext()
+        context._layout = "hello_view"
+        published = DummyPublished(context)
+
+        etag = Layout(published, request)
+
+        self.assertEqual("hello_view", etag())
