@@ -199,7 +199,11 @@ class TestProfileWithCaching(unittest.TestCase):
         browser.open(self.portal["f1"]["d1"].absolute_url())
         # This should be a 304 response
         self.assertEqual("304 Not Modified", browser.headers["Status"])
-        self.assertEqual("", browser.contents)
+        # We MUST have an empty byte, not text, otherwise it is an indication that we
+        # get a possibly wrong Content-Type in a 304 status.
+        # See https://github.com/zopefoundation/Zope/issues/1089.
+        self.assertEqual(b"", browser.contents)
+        self.assertFalse(browser.headers["Content-Type"])
 
         # Request the anonymous folder
         now = stable_now()
@@ -274,7 +278,8 @@ class TestProfileWithCaching(unittest.TestCase):
         )
         # This should be a 304 response
         self.assertEqual("304 Not Modified", browser.headers["Status"])
-        self.assertEqual("", browser.contents)
+        self.assertEqual(b"", browser.contents)
+        self.assertFalse(browser.headers["Content-Type"])
 
         # Edit the page to update the etag
         testText2 = "Testing... body two"
@@ -369,7 +374,8 @@ class TestProfileWithCaching(unittest.TestCase):
         )
         # This should be a 304 response
         self.assertEqual("304 Not Modified", browser.headers["Status"])
-        self.assertEqual("", browser.contents)
+        self.assertEqual(b"", browser.contents)
+        self.assertFalse(browser.headers["Content-Type"])
 
         # Request the authenticated rss feed
         now = stable_now()
@@ -504,7 +510,8 @@ class TestProfileWithCaching(unittest.TestCase):
         )
         # This should be a 304 response
         self.assertEqual("304 Not Modified", browser.headers["Status"])
-        self.assertEqual("", browser.contents)
+        self.assertEqual(b"", browser.contents)
+        self.assertFalse(browser.headers["Content-Type"])
 
         # Request an image scale
         now = stable_now()
@@ -560,7 +567,8 @@ class TestProfileWithCaching(unittest.TestCase):
         )
         # This should be a 304 response
         self.assertEqual("304 Not Modified", browser.headers["Status"])
-        self.assertEqual("", browser.contents)
+        self.assertEqual(b"", browser.contents)
+        self.assertFalse(browser.headers["Content-Type"])
 
         # Request a large datafile (over 64K) to test files that use
         # the "response.write()" function to initiate a streamed response.
